@@ -1,6 +1,7 @@
 <?php
 
 namespace Drupal\Component\Datetime;
+
 use Drupal\Component\Utility\ToStringTrait;
 
 /**
@@ -22,17 +23,30 @@ use Drupal\Component\Utility\ToStringTrait;
  * errors are. This is less disruptive than allowing datetime exceptions
  * to abort processing. The calling script can decide what to do about
  * errors using hasErrors() and getErrors().
+ *
+ * @method $this add(\DateInterval $interval)
+ * @method static array getLastErrors()
+ * @method $this modify(string $modify)
+ * @method $this setDate(int $year, int $month, int $day)
+ * @method $this setISODate(int $year, int $week, int $day = 1)
+ * @method $this setTime(int $hour, int $minute, int $second = 0, int $microseconds = 0)
+ * @method $this setTimestamp(int $unix_timestamp)
+ * @method $this setTimezone(\DateTimeZone $timezone)
+ * @method $this sub(\DateInterval $interval)
+ * @method int getOffset()
+ * @method int getTimestamp()
+ * @method \DateTimeZone getTimezone()
  */
 class DateTimePlus {
 
   use ToStringTrait;
 
-  const FORMAT   = 'Y-m-d H:i:s';
+  const FORMAT = 'Y-m-d H:i:s';
 
   /**
    * A RFC7231 Compliant date.
    *
-   * http://tools.ietf.org/html/rfc7231#section-7.1.1.1
+   * @see http://tools.ietf.org/html/rfc7231#section-7.1.1.1
    *
    * Example: Sun, 06 Nov 1994 08:49:37 GMT
    */
@@ -40,6 +54,8 @@ class DateTimePlus {
 
   /**
    * An array of possible date parts.
+   *
+   * @var string[]
    */
   protected static $dateParts = [
     'year',
@@ -52,41 +68,57 @@ class DateTimePlus {
 
   /**
    * The value of the time value passed to the constructor.
+   *
+   * @var string
    */
   protected $inputTimeRaw = '';
 
   /**
    * The prepared time, without timezone, for this date.
+   *
+   * @var string
    */
   protected $inputTimeAdjusted = '';
 
   /**
    * The value of the timezone passed to the constructor.
+   *
+   * @var string
    */
   protected $inputTimeZoneRaw = '';
 
   /**
    * The prepared timezone object used to construct this date.
+   *
+   * @var string
    */
   protected $inputTimeZoneAdjusted = '';
 
   /**
    * The value of the format passed to the constructor.
+   *
+   * @var string
    */
   protected $inputFormatRaw = '';
 
   /**
    * The prepared format, if provided.
+   *
+   * @var string
    */
   protected $inputFormatAdjusted = '';
 
   /**
    * The value of the language code passed to the constructor.
+   *
+   * @var string|null
    */
   protected $langcode = NULL;
 
   /**
    * An array of errors encountered when creating this date.
+   *
+   * @var string[]
    */
   protected $errors = [];
 
@@ -100,15 +132,16 @@ class DateTimePlus {
   /**
    * Creates a date object from an input date object.
    *
-   * @param \DateTime $datetime
+   * @param \DateTimeInterface $datetime
    *   A DateTime object.
    * @param array $settings
-   *   @see __construct()
+   *   (optional) A keyed array for settings, suitable for passing on to
+   *   __construct().
    *
    * @return static
    *   A new DateTimePlus object.
    */
-  public static function createFromDateTime(\DateTime $datetime, $settings = []) {
+  public static function createFromDateTime(\DateTimeInterface $datetime, $settings = []) {
     return new static($datetime->format(static::FORMAT), $datetime->getTimezone(), $settings);
   }
 
@@ -157,9 +190,11 @@ class DateTimePlus {
    * @param int $timestamp
    *   A UNIX timestamp.
    * @param mixed $timezone
-   *   @see __construct()
+   *   (optional) \DateTimeZone object, time zone string or NULL. See
+   *   __construct() for more details.
    * @param array $settings
-   *   @see __construct()
+   *   (optional) A keyed array for settings, suitable for passing on to
+   *   __construct().
    *
    * @return static
    *   A new DateTimePlus object.
@@ -185,11 +220,14 @@ class DateTimePlus {
    *   any other specialized input with a known format. If provided the
    *   date will be created using the createFromFormat() method.
    *   @see http://php.net/manual/datetime.createfromformat.php
-   * @param mixed $time
-   *   @see __construct()
+   * @param string $time
+   *   String representing the time.
    * @param mixed $timezone
-   *   @see __construct()
+   *   (optional) \DateTimeZone object, time zone string or NULL. See
+   *   __construct() for more details.
    * @param array $settings
+   *   (optional) A keyed array for settings, suitable for passing on to
+   *   __construct(). Supports an additional key:
    *   - validate_format: (optional) Boolean choice to validate the
    *     created date using the input format. The format used in
    *     createFromFormat() allows slightly different values than format().
@@ -197,7 +235,6 @@ class DateTimePlus {
    *     possible to a validation step to confirm that the date created
    *     from a format string exactly matches the input. This option
    *     indicates the format can be used for validation. Defaults to TRUE.
-   *   @see __construct()
    *
    * @return static
    *   A new DateTimePlus object.
@@ -215,32 +252,26 @@ class DateTimePlus {
     // Tries to create a date from the format and use it if possible.
     // A regular try/catch won't work right here, if the value is
     // invalid it doesn't return an exception.
-    $datetimeplus = new static('', $timezone, $settings);
+    $datetime_plus = new static('', $timezone, $settings);
 
-    $date = \DateTime::createFromFormat($format, $time, $datetimeplus->getTimezone());
+    $date = \DateTime::createFromFormat($format, $time, $datetime_plus->getTimezone());
     if (!$date instanceof \DateTime) {
       throw new \InvalidArgumentException('The date cannot be created from a format.');
     }
     else {
+      $datetime_plus->setTimestamp($date->getTimestamp());
+      $datetime_plus->setTimezone($date->getTimezone());
+
       // Functions that parse date is forgiving, it might create a date that
       // is not exactly a match for the provided value, so test for that by
       // re-creating the date/time formatted string and comparing it to the input. For
       // instance, an input value of '11' using a format of Y (4 digits) gets
       // created as '0011' instead of '2011'.
-      if ($date instanceof DateTimePlus) {
-        $test_time = $date->format($format, $settings);
-      }
-      elseif ($date instanceof \DateTime) {
-        $test_time = $date->format($format);
-      }
-      $datetimeplus->setTimestamp($date->getTimestamp());
-      $datetimeplus->setTimezone($date->getTimezone());
-
-      if ($settings['validate_format'] && $test_time != $time) {
+      if ($settings['validate_format'] && $date->format($format) != $time) {
         throw new \UnexpectedValueException('The created date does not match the input value.');
       }
     }
-    return $datetimeplus;
+    return $datetime_plus;
   }
 
   /**
@@ -254,7 +285,7 @@ class DateTimePlus {
    *   parameter and the current timezone are ignored when the $time parameter
    *   either is a UNIX timestamp (e.g. @946684800) or specifies a timezone
    *   (e.g. 2010-01-28T15:00:00+02:00).
-   *   @see http://php.net/manual/en/datetime.construct.php
+   *   @see http://php.net/manual/datetime.construct.php
    * @param array $settings
    *   (optional) Keyed array of settings. Defaults to empty array.
    *   - langcode: (optional) String two letter language code used to control
@@ -305,16 +336,36 @@ class DateTimePlus {
    * Implements the magic __call method.
    *
    * Passes through all unknown calls onto the DateTime object.
+   *
+   * @param string $method
+   *   The method to call on the decorated object.
+   * @param array $args
+   *   Call arguments.
+   *
+   * @return mixed
+   *   The return value from the method on the decorated object. If the proxied
+   *   method call returns a DateTime object, then return the original
+   *   DateTimePlus object, which allows function chaining to work properly.
+   *   Otherwise, the value from the proxied method call is returned.
+   *
+   * @throws \Exception
+   *   Thrown when the DateTime object is not set.
+   * @throws \BadMethodCallException
+   *   Thrown when there is no corresponding method on the DateTime object to
+   *   call.
    */
-  public function __call($method, $args) {
+  public function __call($method, array $args) {
     // @todo consider using assert() as per https://www.drupal.org/node/2451793.
     if (!isset($this->dateTimeObject)) {
       throw new \Exception('DateTime object not set.');
     }
     if (!method_exists($this->dateTimeObject, $method)) {
-      throw new \BadMethodCallException(sprintf('Call to undefined method %s::%s()', get_class($this), $method));
+      throw new \BadMethodCallException(sprintf('Call to undefined method %s::%s()', static::class, $method));
     }
-    return call_user_func_array([$this->dateTimeObject, $method], $args);
+
+    $result = call_user_func_array([$this->dateTimeObject, $method], $args);
+
+    return $result === $this->dateTimeObject ? $this : $result;
   }
 
   /**
@@ -348,7 +399,7 @@ class DateTimePlus {
    */
   public static function __callStatic($method, $args) {
     if (!method_exists('\DateTime', $method)) {
-      throw new \BadMethodCallException(sprintf('Call to undefined method %s::%s()', get_called_class(), $method));
+      throw new \BadMethodCallException(sprintf('Call to undefined method %s::%s()', static::class, $method));
     }
     return call_user_func_array(['\DateTime', $method], $args);
   }
@@ -431,7 +482,6 @@ class DateTimePlus {
     return $format;
   }
 
-
   /**
    * Examines getLastErrors() to see what errors to report.
    *
@@ -466,7 +516,7 @@ class DateTimePlus {
    *   otherwise.
    */
   public function hasErrors() {
-    return (boolean) count($this->errors);
+    return (bool) count($this->errors);
   }
 
   /**
@@ -574,11 +624,10 @@ class DateTimePlus {
     $valid_date = FALSE;
     $valid_time = TRUE;
     // Check for a valid date using checkdate(). Only values that
-    // meet that test are valid.
-    if (array_key_exists('year', $array) && array_key_exists('month', $array) && array_key_exists('day', $array)) {
-      if (@checkdate($array['month'], $array['day'], $array['year'])) {
-        $valid_date = TRUE;
-      }
+    // meet that test are valid. An empty value, either a string or a 0, is not
+    // a valid value.
+    if (!empty($array['year']) && !empty($array['month']) && !empty($array['day'])) {
+      $valid_date = checkdate($array['month'], $array['day'], $array['year']);
     }
     // Testing for valid time is reversed. Missing time is OK,
     // but incorrect values are not.
@@ -591,6 +640,7 @@ class DateTimePlus {
               $valid_time = FALSE;
             }
             break;
+
           case 'minute':
           case 'second':
           default:
@@ -625,7 +675,7 @@ class DateTimePlus {
    * Formats the date for display.
    *
    * @param string $format
-   *   A format string using either PHP's date().
+   *   Format accepted by date().
    * @param array $settings
    *   - timezone: (optional) String timezone name. Defaults to the timezone
    *     of the date object.
@@ -658,4 +708,25 @@ class DateTimePlus {
     return $value;
   }
 
+  /**
+   * Sets the default time for an object built from date-only data.
+   *
+   * The default time for a date without time can be anything, so long as it is
+   * consistently applied. If we use noon, dates in most timezones will have the
+   * same value for in both the local timezone and UTC.
+   */
+  public function setDefaultDateTime() {
+    $this->dateTimeObject->setTime(12, 0, 0);
+  }
+
+  /**
+   * Gets a clone of the proxied PHP \DateTime object wrapped by this class.
+   *
+   * @return \DateTime
+   *   A clone of the wrapped PHP \DateTime object.
+   */
+  public function getPhpDateTime() {
+    return clone $this->dateTimeObject;
+  }
+
 }
